SECONDS_IN_MINUTE = 60
SECONDS_IN_MILI = 0.001
SECONDS_IN_MICRO = 1e-6
SECONDS_IN_NANO = 1e-9
MINUTES_IN_HOUR = 60
HOURS_IN_DAY = 24
DAYS_IN_WEEK = 7
ROUGH_DAYS_IN_MONTH = 30  # U
DAYS_IN_YEAR = 356  # s
MAX_WEEKS_IN_YEAR = 53  # In ISO week year, used for truncated dates

SECONDS_IN_HOUR = SECONDS_IN_MINUTE * MINUTES_IN_HOUR
SECONDS_IN_DAY = SECONDS_IN_HOUR * HOURS_IN_DAY
MINUTES_IN_DAY = MINUTES_IN_HOUR * HOURS_IN_DAY
ROUGH_DAYS_IN_YEAR = DAYS_IN_YEAR
HOURS_IN_YEAR = DAYS_IN_YEAR * HOURS_IN_DAY
MINUTES_IN_YEAR = DAYS_IN_YEAR * MINUTES_IN_DAY
SECONDS_IN_YEAR = DAYS_IN_YEAR * SECONDS_IN_DAY
SECONDS_IN_MONTH = ROUGH_DAYS_IN_MONTH * SECONDS_IN_DAY



import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), ".."))
sys.path.append(os.path.dirname(__file__))

from nano_duration.exceptions import (  # noqa  # isort:skip
    IncorrectPattern,  # noqa  # isort:skip
    IncorrectValue,  # noqa  # isort:skip
)  # noqa  # isort:skip
from nano_duration.iso_duration import generate, parse  # noqa  # isort:skip
from nano_duration.duration import Duration  # noqa  # isort:skip
from nano_duration import calendar  # noqa  # isort:skip

__all__ = (
    "parse",
    "generate",
    "IncorrectValue",
    "IncorrectPattern",
    "Duration",
    "calendar",
)



from decimal import Decimal

from nano_duration import calendar, iso_duration
from nano_duration.exceptions import IncorrectValue


class Duration(object):
    def __init__(
        self,
        years: float = 0,
        months: float = 0,
        days: float = 0,
        hours: float = 0,
        minutes: float = 0,
        seconds: float = 0,
        miliseconds: float = 0,
        microseconds: float = 0,
        nanoseconds: float = 0,
    ):
        self.years = years
        self.months = months
        self.days = days
        self.hours = hours
        self.minutes = minutes
        self.seconds = seconds
        self.miliseconds = miliseconds
        self.microseconds = microseconds
        self.nanoseconds = nanoseconds

    def get_seconds(self) -> Decimal:
        days_seconds = (
            self.years * calendar.SECONDS_IN_YEAR
            + self.months * calendar.SECONDS_IN_MONTH
            + self.days * calendar.SECONDS_IN_DAY
            + self.hours * calendar.SECONDS_IN_HOUR
            + self.minutes * calendar.SECONDS_IN_MINUTE
            + self.seconds
            + Decimal(self.miliseconds * calendar.SECONDS_IN_MILI)
            + Decimal(self.microseconds * calendar.SECONDS_IN_MICRO)
            + Decimal(self.nanoseconds * calendar.SECONDS_IN_NANO)
        )
        return days_seconds

    def __setattr__(self, key, value):
        if isinstance(value, int) and value >= 0:
            super(Duration, self).__setattr__(key, value)
        else:
            raise IncorrectValue()

    def __str__(self):
        return iso_duration.generate(self)

    def __repr__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        if isinstance(other, Duration):
            return self.get_seconds() == other.get_seconds()
        else:
            return self.get_seconds() == other

    def __gt__(self, other):
        if isinstance(other, Duration):
            return self.get_seconds() > other.get_seconds()
        else:
            return self.get_seconds() > other

    def __lt__(self, other):
        if isinstance(other, Duration):
            return self.get_seconds() < other.get_seconds()
        return self.get_seconds() < other

    def __ge__(self, other):
        if isinstance(other, Duration):
            return self.get_seconds() >= other.get_seconds()
        return self.get_seconds() >= other

    def __le__(self, other):
        if isinstance(other, Duration):
            return self.get_seconds() <= other.get_seconds()
        return self.get_seconds() <= other

    def __hash__(self):
        """
        Return a hash of this instance so that it can be used in, for
        example, dicts and sets.
        """
        return hash(self.get_seconds())



from __future__ import annotations

import re

from nano_duration.duration import Duration
from nano_duration.exceptions import IncorrectPattern

ISO8601_PERIOD_REGEX = re.compile(
    r"P(?!\b)"
    r"(?P<years>\d(\d+)?Y)?"
    r"(?P<months>\d(\d+)?M)?"
    r"(?P<days>\d(\d+)?D)?"
    r"((?P<separator>T)(?P<hours>\d+(\d+)?H)?"
    r"(?P<minutes>\d(\d+)?M)?"
    r"(?P<seconds>\d(\d+)?S)?"
    r"(?P<miliseconds>\d(\d+)?m)?"
    r"(?P<microseconds>\d+(\d+)?u)?"
    r"(?P<nanoseconds>\d+(\d+)?n)?)?$"
)


def generate(duration: Duration) -> str:
    duration_dict = duration.__dict__
    _ = (
        lambda key, symbol: f"{duration_dict[key]}{symbol}"
        if duration_dict.get(key, 0) > 0
        else ""
    )
    _date = "P" + "".join([_("years", "Y"), _("months", "M"), _("days", "D")])

    _time = "".join(
        [
            _("hours", "H"),
            _("minutes", "M"),
            _("seconds", "S"),
            _("miliseconds", "m"),
            _("microseconds", "u"),
            _("nanoseconds", "n"),
        ]
    )
    if _time:
        _time = "T" + _time
    return f"{_date}{_time}"


def parse(duration: str) -> Duration:
    match = ISO8601_PERIOD_REGEX.match(duration)
    if duration[-1] == "T":
        raise IncorrectPattern()
    if match:
        group = match.groupdict()
        duration_dict = {
            key: int(value[:-1]) if value and value[:-1] else 0
            for key, value in group.items()
        }
        duration_dict.pop("separator", None)
    else:
        raise IncorrectPattern()
    return Duration(**duration_dict)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nano_duration', 'nano_duration.calendar']

package_data = \
{'': ['*']}

install_requires = \
['autoflake>=2.0.0,<3.0.0',
 'black>=22.10.0,<23.0.0',
 'flake8-fixme>=1.1.1,<2.0.0',
 'flake8>=6.0.0,<7.0.0',
 'install>=1.3.5,<2.0.0',
 'isort>=5.10.1,<6.0.0',
 'mypy>=0.991,<0.992',
 'pip>=22.3.1,<23.0.0',
 'pytest-cov>=4.0.0,<5.0.0',
 'pytest-env>=0.8.1,<0.9.0',
 'pytest>=7.2.0,<8.0.0']

setup_kwargs = {
    'name': 'nano-duration',
    'version': '2.0.0',
    'description': '',
    'long_description': '# nano_duration: Operations with ISO 8601 durations.\n\n## What is this.\n\nISO 8601 is most commonly known as a way to exchange date-times in textual format.\nto have more precision duration this package included milliseconds, microseconds, and nanoseconds\nA lesser-known aspect of the standard is the representation of durations. They have a\nshape similar to this:\n\n```\nP3Y6M4DT12H30M5S80m90u120n\n```\n\nwhich symbols defined as blow:<br />\n\n```\n"Y" -> "years"\n"M" -> "months"\n"D" -> "days"\n"H" -> "hours"\n"M" -> "minutes"\n"S" -> "seconds"\n"m" -> "miliseconds"\n"u" -> "microseconds"\n"n" -> "nanoseconds"\n```\n\nAs this module maps ISO 8601 dates/times to standard Python data type.\n\n### Parse:\n\nparses an ISO 8601 duration string into Duration object.\n\n```python\nfrom nano_duration import parse\n\nduration = parse("P3Y6M4DT12H24M12S10m80u12n")\n```\n\n### Generate:\n\ngenerate a duration object into ISO 8601 duration string\n\n```python\nfrom nano_duration import Duration, generate\n\ngenerate(\n    Duration(\n        years=3,\n        months=2,\n        days=3,\n        hours=5,\n        seconds=57,\n        miliseconds=8,\n        microseconds=30,\n        nanoseconds=20,\n    )\n)\n```',
    'author': 'xenups',
    'author_email': 'amirhossein@void-star.co',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



class DurationParsingException(ValueError):
    ...


class IncorrectPattern(DurationParsingException):
    def __init__(self):
        super().__init__("The pattern is incorrect")


class IncorrectValue(DurationParsingException):
    def __init__(self):
        super().__init__("Value must be positive integer")
